use crate::yt_meta::{self, get_vid_duration};
use based::{get_pg, request::api::ToAPI};
use chrono::Utc;
use serde::{Deserialize, Serialize};
use serde_json::json;
use sqlx::prelude::FromRow;
use std::path::PathBuf;

pub trait LogAndIgnore {
    fn log_and_ignore(self, msg: &str);
}

impl<T, E: std::fmt::Debug> LogAndIgnore for Result<T, E> {
    /// Handles the result by ignoring and logging it if it contains an error.
    ///
    /// If the result is `Ok`, does nothing.
    /// If the result is `Err(e)`
    /// logs the message provided (`msg`) along with the error.
    fn log_and_ignore(self, msg: &str) {
        match self {
            Ok(_) => {}
            Err(e) => {
                log::error!("{msg} : {:?}", e);
            }
        }
    }
}

#[derive(Debug, Clone, Serialize, Deserialize, FromRow)]
pub struct YouTubeMeta {
    pub id: String,
    pub title: String,
    pub description: String,
    pub uploader_name: String,
    pub uploader_id: String,
    pub views: i32,
    pub upload_date: chrono::NaiveDate,
}

impl YouTubeMeta {
    pub async fn get(id: &str) -> Option<Self> {
        sqlx::query_as("SELECT * FROM youtube_meta WHERE id = $1")
            .bind(id)
            .fetch_optional(get_pg!())
            .await
            .unwrap()
    }

    pub async fn tags(&self) -> Vec<String> {
        let res: Vec<(String,)> =
            sqlx::query_as("SELECT tag FROM youtube_meta_tags WHERE youtube_id = $1")
                .bind(&self.id)
                .fetch_all(get_pg!())
                .await
                .unwrap();

        res.into_iter().map(|x| x.0).collect()
    }

    pub async fn categories(&self) -> Vec<String> {
        let res: Vec<(String,)> =
            sqlx::query_as("SELECT category FROM youtube_meta_categories WHERE youtube_id = $1")
                .bind(&self.id)
                .fetch_all(get_pg!())
                .await
                .unwrap();

        res.into_iter().map(|x| x.0).collect()
    }
}

#[derive(Debug, Clone, Serialize, Deserialize, FromRow)]
pub struct Video {
    pub id: uuid::Uuid,
    pub directory: String,
    pub path: String,
    pub duration: f64,
    pub title: String,
    pub date_added: chrono::DateTime<Utc>,
    youtube_id: Option<String>,
}

// Video Init
impl Video {
    pub async fn get(id: &uuid::Uuid) -> Option<Self> {
        sqlx::query_as("SELECT * FROM videos WHERE id = $1")
            .bind(id)
            .fetch_optional(get_pg!())
            .await
            .unwrap()
    }

    pub async fn youtube_meta(&self) -> Option<YouTubeMeta> {
        if let Some(meta) = self.youtube_id.as_ref().map(|id| YouTubeMeta::get(id)) {
            return meta.await;
        }
        None
    }

    pub async fn has_path(path: &str) -> bool {
        sqlx::query("SELECT id FROM videos WHERE path = $1")
            .bind(path)
            .fetch_optional(get_pg!())
            .await
            .unwrap()
            .is_some()
    }

    pub async fn insert_path_to_db(v: &PathBuf) -> Option<Self> {
        let db = get_pg!();

        log::info!("Add {v:?} to library");
        let file_name = v.file_stem().unwrap().to_str().unwrap().to_owned();
        let dir = v
            .parent()
            .unwrap()
            .file_name()
            .unwrap()
            .to_str()
            .unwrap()
            .to_owned();

        let mut tx = db.begin().await.unwrap();

        let vid_duration = get_vid_duration(v).unwrap();

        if let Some(meta) = yt_meta::get_youtube_metadata(v) {
            sqlx::query("INSERT INTO youtube_meta (id, title, description, uploader_name, uploader_id, views, upload_date) VALUES ($1, $2, $3, $4, $5, $6, $7) ON CONFLICT (id) DO UPDATE SET title = EXCLUDED.title, description = EXCLUDED.description, uploader_name = EXCLUDED.uploader_name, uploader_id = EXCLUDED.uploader_id, views = EXCLUDED.views, upload_date = EXCLUDED.upload_date;")
                .bind(&meta.youtube_id().unwrap())
                .bind(&meta.title())
                .bind(&meta.description().unwrap())
                .bind(&meta.uploader_name().unwrap())
                .bind(&meta.uploader_id().unwrap())
                .bind(&meta.views().unwrap())
                .bind(&meta.upload_date())
                .execute(&mut *tx).await.unwrap();

            let vid = sqlx::query_as("INSERT INTO videos (directory, path, duration, title, youtube_id) VALUES ($1, $2, $3, $4, $5) RETURNING *")
                .bind(&dir)
                .bind(v.to_str().unwrap())
                .bind(vid_duration)
                .bind(meta.title())
                .bind(meta.youtube_id().unwrap())
                .fetch_one(&mut *tx).await.unwrap();

            for cat in meta.categories().unwrap() {
                sqlx::query(
                    "INSERT INTO youtube_meta_categories (youtube_id, category) VALUES ($1, $2) ON CONFLICT DO NOTHING",
                )
                .bind(meta.youtube_id().unwrap())
                .bind(cat)
                .execute(&mut *tx)
                .await
                .log_and_ignore("Category mapping already exists");
            }

            if let Some(tags) = meta.tags() {
                for tag in tags {
                    sqlx::query("INSERT INTO youtube_meta_tags (youtube_id, tag) VALUES ($1, $2) ON CONFLICT DO NOTHING")
                        .bind(&meta.youtube_id().unwrap())
                        .bind(&tag)
                        .execute(&mut *tx)
                        .await
                        .log_and_ignore("YT Tag mapping already exists");
                }
            }

            tx.commit().await.unwrap();

            return Some(vid);
        }

        let vid = sqlx::query_as(
            "INSERT INTO videos (directory, path, duration, title) VALUES ($1, $2, $3, $4) RETURNING *",
        )
        .bind(dir)
        .bind(v.to_str().unwrap())
        .bind(vid_duration)
        .bind(file_name)
        .fetch_one(&mut *tx)
        .await
        .unwrap();

        tx.commit().await.unwrap();

        Some(vid)
    }
}

impl ToAPI for Video {
    async fn api(&self) -> serde_json::Value {
        let yt_meta = if let Some(yt_meta) = &self.youtube_id {
            YouTubeMeta::get(yt_meta).await
        } else {
            None
        };

        json!({
            "id": &self.id,
            "directory": self.directory,
            "duration": self.duration,
            "title": self.title,
            "yt": yt_meta
        })
    }
}
