use crate::library::Video;
use based::ui::components::prelude::Avatar;
use based::ui::components::prelude::*;
use based::ui::primitives::flex::Row;
use based::ui::primitives::Optional;
use based::ui::wrapper::HoverWrapper;
use based::ui::{prelude::*, UIWidget};
use based::{
    auth::User,
    format::{format_date, format_number, format_seconds_to_hhmmss},
    request::{RequestContext, StringResponse},
};
use maud::{html, PreEscaped, Render};

pub async fn render_page(
    shell: &Shell,
    ctx: RequestContext,
    content: PreEscaped<String>,
    title: &str,
    user: Option<User>,
) -> StringResponse {
    shell
        .extend()
        .with_navbar(
            NavBar("WatchDogs")
                .icon(Sized(
                    ScreenValue::_8,
                    ScreenValue::_8,
                    Rounded(Image("/favicon").alt("Logo")).size(Size::Medium),
                ))
                .extra(Optional(user.as_ref(), |user| {
                    // TODO : profile pictures
                    DropDown(
                        Avatar("", &user.username).use_initials(),
                        DropdownMenu(vec![
                            DropdownMenuEntry(
                                "/account",
                                Row(vec![
                                    MaterialIcon("account_circle"),
                                    Text("Account").medium().render(),
                                ])
                                .gap(ScreenValue::_2)
                                .items_center()
                                .justify(Justify::Start),
                            ),
                            DropdownMenuEntry(
                                "/history",
                                Row(vec![
                                    MaterialIcon("history"),
                                    Text("Video History").medium().render(),
                                ])
                                .gap(ScreenValue::_2)
                                .items_center()
                                .justify(Justify::Start),
                            ),
                        ]),
                    )
                }))
                .no_dropdown(),
        )
        .render_page(content, title, ctx)
        .await
}

#[allow(non_snake_case)]
pub fn HoverScaleAnimation<T: UIWidget + 'static>(inner: T) -> HoverWrapper {
    Hover(Animated(ZIndex::Five(Scale(1.02, Nothing()))).scope(Scope::All)).on(inner)
}

#[allow(non_snake_case)]
pub fn Title<T: UIWidget + 'static>(title: T) -> PreEscaped<String> {
    Margin(
        Paragraph(title)
            .align(TextAlignment::Center)
            ._4xl()
            .extrabold()
            .line_height(LineHeight::Tight),
    )
    .top(ScreenValue::_6)
    .bottom(ScreenValue::_2)
    .render()
}

#[allow(non_snake_case)]
pub async fn VerticalVideoGrid(videos: &mut [Video]) -> PreEscaped<String> {
    let video_elements = html! {
        @for mut vid in videos {
            ( video_element(&mut vid).await );
        };
    };

    VerticalVideoGridRaw(video_elements)
}

#[allow(non_snake_case)]
pub fn VerticalVideoGridRaw<T: UIWidget + 'static>(inner: T) -> PreEscaped<String> {
    Screen::large(Grid(Nothing()).columns(GridAmount::_3).gap(ScreenValue::_6))
        .on(Padding(Margin(inner).bottom(ScreenValue::_4)).all(ScreenValue::_6))
        .render()
}

pub async fn video_element_wide(video: &Video) -> PreEscaped<String> {
    let yt_meta = video.youtube_meta().await;
    Flex(Margin(Width(ScreenValue::full,
        Hover(Background(Nothing()).color(Gray::_800)).on(
        Background(
            Rounded(
            Shadow::large(
        Link(&format!("/watch?v={}", video.id),
                Div().vanish().push(
                            FlexGrow(Strategy::NoShrink,
                                Position(PositionKind::Relative,
                                    Div().vanish().push(
                                        html! {
                                            img width="480" src=(format!("/video/thumbnail?v={}", video.id)) class="aspect-video h-32 w-48 rounded-l-md object-cover";
                                        }
                                    ).push(
                                        Position(PositionKind::Absolute,
                                            Background(
                                                Rounded(
                                                    Opacity(0.90,
                                                        Padding(
                                                            Span(
&format_seconds_to_hhmmss(video.duration)
                                                            ).white().xs()
                                                        ).x(ScreenValue::_2).y(ScreenValue::_1)
                                                    )
                                                ).size(Size::Small)
                                            ).color(Colors::Black)
                                        ).bottom(8).right(8)
                                    )
                                )
                            )
                ).push(
                            Flex(
                                FlexGrow(Strategy::Grow,
                                    Margin(
                                        MaxHeight(ScreenValue::_32,
                                                    Div().vanish().push(
                                                        Margin(Text(&video.title).base_size().semibold().max_lines(LineClamp::_3).title(&video.title).align(TextAlignment::Start)
                                            ).bottom(ScreenValue::_1)
                                                    ).push_some(yt_meta, |meta| {
                                                        Flex(
                                                            FlexGrow(Strategy::Grow,
                                                                Margin(
                                                                    Padding(
                                                                                Div().vanish().push(Context(
                                                                                 Paragraph(
                                                                                    Margin(
                                                                                        Paragraph(Div().vanish().push(
                                                                                            Span(&meta.uploader_name).medium()
                                                                                        ).push(
                                                                                            Span(" - ")
                                                                                        ).push(
                                                                                            Span(&format_date(&meta.upload_date)).normal()
                                                                                        )).xs().align(TextAlignment::Start)
                                                                                    ).bottom(ScreenValue::_2)
                                                                                ).sm().color(&Gray::_400).align(TextAlignment::Start)
                                                                                )).push(Context(
                                                                                    Paragraph(
                                                                                Div().vanish().push(
                                                                                    Span(&format_number(meta.views))
                                                                                ).push(
                                                                                    Span(" views")
                                                                                )
                                                                            ).xs().color(&Gray::_400).align(TextAlignment::Start))
                                                                                )
                                                                    ).y(ScreenValue::_1)
                                                                )
                                                            )
                                                        ).direction(Direction::Column)
                                                    })
                                        )
                                    ).left(ScreenValue::_2)
                                )
                            ).direction(Direction::Column)
                )

        ))).size(Size::Large)).color(Gray::_900))


)).y(ScreenValue::_4)).items_center().render()
}

pub fn video_thumbnail_with_time(video: &Video) -> PreEscaped<String> {
    Position(
        PositionKind::Relative,
        Div()
            .push(Context(ObjectFit::Cover(Width(
                ScreenValue::full,
                Height(
                    ScreenValue::auto,
                    Aspect::Video(
                        Rounded(Image(&format!("/video/thumbnail?v={}", video.id)))
                            .size(Size::Large)
                            .side(Side::Top),
                    ),
                ),
            ))))
            .push(Context(
                Position(
                    PositionKind::Absolute,
                    Background(
                        Padding(
                            Rounded(Opacity(
                                0.90,
                                Span(&format_seconds_to_hhmmss(video.duration)).white().xs(),
                            ))
                            .size(Size::Small),
                        )
                        .x(ScreenValue::_2)
                        .y(ScreenValue::_1),
                    )
                    .color(Colors::Black),
                )
                .bottom(8)
                .right(8),
            )),
    )
    .render()
}

pub async fn video_element(video: &mut Video) -> PreEscaped<String> {
    HoverScaleAnimation(
        Margin(Context(MaxWidth(
            ScreenValue::_90,
            MaxHeight(
                ScreenValue::_60,
                Aspect::Video(Link(
                    &format!("/watch?v={}", video.id),
                    Context(
                        Hover(Background(Nothing()).color(Gray::_800)).on(Background(
                            Shadow::large(
                                Rounded(
                                    Div().push(video_thumbnail_with_time(&video)).push(Context(
                                        Rounded(Shadow::large(
                                            Padding(
                                                Text(&video.title)
                                                    .base_size()
                                                    .semibold()
                                                    .align(TextAlignment::Start)
                                                    .title(&video.title)
                                                    .max_lines(LineClamp::_1)
                                                    .overflow(TextOverflow::Truncate),
                                            )
                                            .y(ScreenValue::_4)
                                            .x(ScreenValue::_2),
                                        ))
                                        .size(Size::Large),
                                    )),
                                )
                                .size(Size::Large),
                            ),
                        )
                        .color(Gray::_900)),
                    ),
                )),
            ),
        )))
        .y(ScreenValue::_6),
    )
    .render()
}

#[allow(non_snake_case)]
pub fn DirectoryBadge(dir: &str) -> PreEscaped<String> {
    Margin(
        Padding(
            Hover(Background(Cursor::Pointer.on(Nothing())).color(Purple::_600)).on(Background(
                Rounded(Link(&format!("/d/{dir}"), Text(dir).white()).use_htmx()).size(Size::Full),
            )
            .color(Purple::_500)),
        )
        .x(ScreenValue::_3)
        .y(ScreenValue::_2),
    )
    .all(ScreenValue::_2)
    .render()
}
