use based::auth::csrf::CSRF;
use based::auth::{Sessions, User};
use based::page;
use based::request::StringResponse;
use based::ui::components::prelude::Avatar;
use based::ui::components::prelude::*;
use based::ui::primitives::flex::Row;
use based::ui::{prelude::*, AttrExtendable};
use based::{auth::MaybeUser, request::RequestContext};
use maud::{html, Render};
use rocket::http::CookieJar;
use rocket::State;
use rocket::{form::Form, get, http::Cookie, post, response::Redirect, FromForm};

use crate::library::history::VideoHistory;
use crate::pages::components::{video_element_wide, Title};

use super::components::render_page;

#[get("/login")]
pub async fn login(ctx: RequestContext, user: MaybeUser, shell: &State<Shell>) -> StringResponse {
    page!(
        shell,
        ctx,
        "Login",
        Margin(Width(
            ScreenValue::_80,
            Div().push(Text("Login")._2xl()).push(
                Margin(
                    based::ui::primitives::input::Form::new("/login")
                        .method(FormMethod::POST)
                        .add_input(TextInput("username").placeholder("Username").required())
                        .add_input(
                            TextInput("password")
                                .placeholder("Password")
                                .required()
                                .password()
                        )
                        .add_input(FormSubmitButton("Login").value("Login"))
                )
                .top(ScreenValue::_4)
            )
        ))
        .x(ScreenValue::auto)
        .top(ScreenValue::_6)
    )
}

#[derive(FromForm)]
pub struct LoginForm {
    username: String,
    password: String,
}

#[post("/login", data = "<login_form>")]
pub async fn login_post(login_form: Form<LoginForm>, cookies: &CookieJar<'_>) -> Option<Redirect> {
    let login_data = login_form.into_inner();

    let (session, _) = User::login(&login_data.username, &login_data.password).await?;

    let session_cookie = Cookie::build(("session", session.token.to_string()))
        .path("/") // Set the cookie path to the root so it’s available for the whole app
        .http_only(true) // Make the cookie HTTP only for security
        .max_age(rocket::time::Duration::days(7)) // Set the cookie expiration (7 days in this case)
        .build();

    cookies.add(session_cookie);

    Some(Redirect::to("/"))
}

#[get("/history")]
pub async fn history_page(ctx: RequestContext, user: User, shell: &State<Shell>) -> StringResponse {
    let video_elements = html! {
        @for mut vid in user.history_of(10).await {
            ( video_element_wide(&mut vid).await );
        };
    };

    let content = Div()
        .vanish()
        .push(Title("History"))
        .push(Padding(video_elements).all(ScreenValue::_6))
        .render();

    render_page(&shell, ctx, content, "History", Some(user)).await
}

#[get("/account")]
pub async fn account(ctx: RequestContext, user: User, shell: &State<Shell>) -> StringResponse {
    let content = Width(
        ScreenValue::fit,
        Margin(
            Div()
                .push(
                    Row(vec![
                        Avatar("", &user.username).render(),
                        Text(&user.username)._2xl().bold().render(),
                    ])
                    .gap(ScreenValue::_2),
                )
                .push(Link("/passwd", Button(Text("Change Password")))),
        )
        .top(ScreenValue::_8)
        .x(ScreenValue::auto),
    )
    .render();

    render_page(&shell, ctx, content, "Account", Some(user)).await
}

#[derive(FromForm)]
pub struct PasswordChangeForm {
    password_old: String,
    password_new: String,
    password_new_repeat: String,
    csrf: String,
}

#[post("/passwd", data = "<form>")]
pub async fn change_password_post(form: Form<PasswordChangeForm>, user: User) -> Redirect {
    if form.password_new != form.password_new_repeat {
        return Redirect::to("/passwd");
    }

    if !user.verify_csrf(&form.csrf).await {
        return Redirect::to("/passwd");
    }

    user.passwd(&form.password_old, &form.password_new)
        .await
        .unwrap();

    Redirect::to("/account")
}

#[get("/passwd")]
pub async fn change_password(
    ctx: RequestContext,
    user: User,
    shell: &State<Shell>,
) -> StringResponse {
    let csrf = user.get_csrf().await.to_string();
    let form = based::ui::primitives::input::Form::new("/passwd")
        .method(FormMethod::POST)
        .add_input(
            Margin(
                Text("Change Password")
                    ._2xl()
                    .bold()
                    .align(TextAlignment::Center),
            )
            .bottom(ScreenValue::_6),
        )
        .add_input(
            TextInput("password_old")
                .password()
                .placeholder("Old Password")
                .required(),
        )
        .add_input(
            TextInput("password_new")
                .password()
                .placeholder("New Password")
                .required(),
        )
        .add_input(
            TextInput("password_new_repeat")
                .password()
                .placeholder("Repeat new password")
                .required(),
        )
        .add_input(FormSubmitButton("Change"))
        .add_input(HiddenInput("csrf", &csrf).add_attr("class", "csrf"))
        .render();

    let content = Div().vanish().push(form).render();

    render_page(&shell, ctx, content, "Change Password", Some(user)).await
}
